﻿Imports Microsoft.DirectX.DirectInput

' DirectX DLLs are to be added from GAC 2902 with SpecificVersion=True and CopyLocal=True
' Project/References/Add/Browse/
' C:\WINDOWS\assembly\GAC\Microsoft.DirectX\1.0.2902.0__31bf3856ad364e35\Microsoft.DirectX.dll
' C:\WINDOWS\assembly\GAC\Microsoft.DirectX.DirectInput\1.0.2902.0__31bf3856ad364e35\Microsoft.DirectX.DirectInput.dll

Module Module_Joystick

    Private DEV(0) As Device
    Private JoyState As JoystickState
    Private FirstSlot As Int32
    Private ZeroCentered As Boolean
    Private Reversed As Boolean
    Private byteArray() As Byte

    Friend Sub JoyOpenAll()
        JoyCloseAll()

        Dim gameControllerList As DeviceList
        gameControllerList = Manager.GetDevices(DeviceClass.GameControl, _
                                                EnumDevicesFlags.AttachedOnly)
        If gameControllerList.Count > 0 Then
            ReDim DEV(gameControllerList.Count - 1)
            Dim i As Int32 = 0
            For Each devInstance As DeviceInstance In gameControllerList
                DEV(i) = New Device(devInstance.InstanceGuid)
                DEV(i).SetCooperativeLevel(Form1, _
                                        CooperativeLevelFlags.Background Or _
                                        CooperativeLevelFlags.NonExclusive)
                DEV(i).SetDataFormat(DeviceDataFormat.Joystick)
                DEV(i).Acquire()
                i += 1
            Next
        End If
    End Sub

    Friend Sub JoyCloseAll()
        For i As Int32 = 0 To DEV.Length - 1
            JoyClose(i)
        Next
    End Sub

    Friend Sub JoyClose(ByVal JoyIndex As Int32)
        If DEV(JoyIndex) IsNot Nothing Then
            DEV(JoyIndex).Unacquire()
            DEV(JoyIndex) = Nothing
        End If
        ResetSlotValues()
    End Sub

    Friend Sub JoyControlPanel()
        ' -------------------------------- try to re-connect devices
        JoyOpenAll()
        ' -------------------------------- open panel
        If DEV(0) Is Nothing Then Return
        DEV(0).RunControlPanel()
    End Sub

    Friend Sub JoySetFirstSlot(ByVal n As Int32)
        FirstSlot = n
        ResetSlotValues()
    End Sub

    Friend Sub JoySetRange(ByVal n As Int32)
        ZeroCentered = n = 2
    End Sub

    Friend Sub JoySetReversed(ByVal rev As Boolean)
        Reversed = rev
        ResetSlotValues()
    End Sub

    Friend Sub JoyReadAllDevices()
        For i As Int32 = 0 To DEV.Length - 1
            CopyValuesToSlots(i)
        Next
    End Sub

    Private Function ReadJoyState(ByVal JoyIndex As Int32) As Boolean
        If DEV(JoyIndex) Is Nothing Then Return False
        Try
            JoyState = DEV(JoyIndex).CurrentJoystickState
            Return True
        Catch
            JoyClose(JoyIndex)
            Return False
        End Try
    End Function

    Private Sub CopyValuesToSlots(ByVal JoyIndex As Int32)
        Dim slot As Int32 = 30 * JoyIndex + FirstSlot
        ' ------------------------------------------------------------
        If DEV(JoyIndex) Is Nothing Then Return
        If Not ReadJoyState(JoyIndex) Then Return
        ' ------------------------------------------------------------ Sticks
        If Reversed Then
            Slots.WriteSlot(slot, StickToTheremino(JoyState.X))
            Slots.WriteSlot(slot + 1, StickToTheremino(65535 - JoyState.Y))
            Slots.WriteSlot(slot + 2, StickToTheremino(JoyState.Z))
            Slots.WriteSlot(slot + 3, StickToTheremino(65535 - JoyState.Rz))
        Else
            Slots.WriteSlot(slot, StickToTheremino(JoyState.X))
            Slots.WriteSlot(slot + 1, StickToTheremino(JoyState.Y))
            Slots.WriteSlot(slot + 2, StickToTheremino(JoyState.Z))
            Slots.WriteSlot(slot + 3, StickToTheremino(JoyState.Rz))
        End If
        Slots.WriteSlot(slot + 4, StickToTheremino(JoyState.Rx))
        Slots.WriteSlot(slot + 5, StickToTheremino(JoyState.Ry))
        ' ------------------------------------------------------------ Debug
        'Form1.Text = Slots.ReadSlot(slot + 0).ToString + "  " + _
        '             Slots.ReadSlot(slot + 1).ToString
        ' ------------------------------------------------------------ Pov
        Slots.WriteSlot(slot + 6, _
                        PovToTheremino(JoyState.GetPointOfView(0)))
        ' ------------------------------------------------------------ Buttons
        For i As Int32 = 0 To 11
            Slots.WriteSlot(slot + 10 + i, If(JoyState.GetButtons(i) = 0, 0, 1000))
        Next
    End Sub

    Private Function StickToTheremino(ByVal n As Single) As Single
        If ZeroCentered Then
            n -= 32767
            If n < 300 And n > -300 Then Return 0
            n /= 32.767F
            If n < -1000 Then n = -1000
            If n > 1000 Then n = 1000
            Return n
        Else
            n -= 32767
            If n < 300 And n > -300 Then Return 500
            n /= 65.534F
            n += 500
            If n < 0 Then n = 0
            If n > 1000 Then n = 1000
            Return n
        End If
    End Function

    Private Function PovToTheremino(ByVal n As Int32) As Single
        If n < 0 Then Return 0
        If n = 0 Then Return 360
        Return n / 100.0F
    End Function

    Private Sub ResetSlotValues()
        Dim slot As Int32
        For d As Int32 = 0 To DEV.Length - 1
            slot = d * 30 + FirstSlot
            For i As Int32 = 0 To 3
                Slots.WriteSlot(slot, If(ZeroCentered, 0, 500))
                slot += 1
            Next
            For i As Int32 = 4 To 11
                Slots.WriteSlot(slot, 0)
                slot += 1
            Next
        Next
    End Sub


    ' Control               Meaning     Slots       Slot values
    '                                   (first=1)   
    ' -------------------------------------------------------------------
    ' Left Stick            X, Y        1, 2        -1000 to +1000
    ' Right Stick           Z, Rz       3, 4        -1000 to +1000
    ' Trimmers              Rx, Ry      5, 6        -1000 to +1000
    ' POV Buttons           Pov         7           1 to 360 (0=unpressed)              
    ' Button 1              Btn 1       11          0 or 1000
    ' Button 2              Btn 2       12          0 or 1000
    ' Button 3              Btn 3       13          0 or 1000
    ' Button 4              Btn 4       14          0 or 1000
    ' Left bumper bottom    Btn 5       15          0 or 1000
    ' Right bumper bottom   Btn 6       16          0 or 1000
    ' Left bumper top       Btn 7       17          0 or 1000
    ' Right bumper top      Btn 8       18          0 or 1000
    ' Button "Select"       Btn 9       19          0 or 1000
    ' Button "Start"        Btn 10      20          0 or 1000
    ' Left stick button     Btn 11      21          0 or 1000
    ' Light stick button    Btn 12      22          0 or 1000

    ' If multiple Joysticks are connected then slots are:
    ' Joystick 1       FirstSlot +  0
    ' Joystick 2       FirstSlot + 30
    ' Joystick 3       FirstSlot + 60
    ' Joystick 4       FirstSlot + 90
    ' and so on...

End Module
